@@ -1,18 +1,21 @@
 """Manages the details of the websocket connection. """
 
 import asyncio
-import iterm2.auth
 import os
 import sys
 import traceback
 import typing
 import websockets
+try:
+  import AppKit
+  import iterm2.auth
+  gAppKitAvailable = True
+except:
+  gAppKitAvailable = False
 
 import iterm2.api_pb2
 from iterm2._version import __version__
 
-# Add "unix" to this array to enable unix domain socket support. Experimental.
-
 def _getenv(key):
     """Gets an environment variable safely.
 
@@ -36,7 +39,7 @@ def _headers():
                "x-iterm2-disable-auth-ui": "true"}
     if cookie is not None:
         headers["x-iterm2-cookie"] = cookie
-    else:
+    elif gAppKitAvailable:
         headers["x-iterm2-advisory-name"] = iterm2.auth.get_script_name()
     if key is not None:
         headers["x-iterm2-key"] = key
@@ -313,7 +316,13 @@ class Connection:
         return (int(parts[0]), int(parts[1]))
 
     def _get_connect_coro(self):
-        return self._get_unix_connect_coro()
+        if gAppKitAvailable:
+            path = self._unix_domain_socket_path()
+            exists = os.path.exists(path)
+
+            if exists:
+                return self._get_unix_connect_coro()
+        return self._get_tcp_connect_coro()
 
     def _remove_auth(self):
         # Remove these because they are not re-usable.
@@ -322,13 +331,18 @@ class Connection:
             if var in os.environ:
                 del os.environ[var]
 
-    def _path_to_unix_domain_socket(self):
-        home = os.path.expanduser("~")
-        return os.path.join(home, "Library", "Application Support", "iTerm2", "private", "socket")
+    def _unix_domain_socket_path(self):
+        applicationSupport = os.path.join(
+            AppKit.NSSearchPathForDirectoriesInDomains(
+                AppKit.NSApplicationSupportDirectory,
+                AppKit.NSUserDomainMask,
+                True)[0],
+            "iTerm2")
+        return os.path.join(applicationSupport, "private", "socket")
 
     def _get_unix_connect_coro(self):
         """Experimental: connect with unix domain socket."""
-        path = self._path_to_unix_domain_socket()
+        path = self._unix_domain_socket_path()
         return websockets.client.unix_connect(
             path,
             "ws://localhost/",
@@ -337,6 +351,13 @@ class Connection:
             subprotocols=_subprotocols())
 
 
+    def _get_tcp_connect_coro(self):
+        """Legacy: connect with tcp socket."""
+        return websockets.connect(_uri(),
+                                        ping_interval=None,
+                                        extra_headers=_headers(),
+                                        subprotocols=_subprotocols())
+
     def authenticate(self, force):
         """
         Request a cookie via Applescript.
@@ -349,6 +370,8 @@ class Connection:
             mean either a failure to connect or that there was already a
             (possibly stale) cookie in the environment.
         """
+        if not gAppKitAvailable:
+            return False
         if force:
             self._remove_auth()
         try:
@@ -446,6 +469,11 @@ raising an exception, pass retry=true to run_until_complete()
 or run_forever()
 
 """, file=sys.stderr)
+                    if gAppKitAvailable:
+                        path = self._unix_domain_socket_path()
+                        exists = os.path.exists(path)
+                        if exists:
+                            print(f"If you have downgraded from iTerm2 3.3.11+ to an older version, you must\nmanually delete the file at {path}.\n", file=sys.stderr)
                     done = True
                     raise
             finally:
